@@ -8,7 +8,7 @@ __copyright__ = "(C) 2010 Science and Technology Facilities Council"
 __contact__ = "Philip.Kershaw@stfc.ac.uk"
 __license__ = "BSD - see LICENSE file in top-level directory"
 __contact__ = "Philip.Kershaw@stfc.ac.uk"
-__revision__ = "$Id: result.py 7443 2010-09-03 13:42:43Z pjkersha $"
+__revision__ = "$Id: result.py 8010 2012-01-30 16:24:06Z rwilkinson $"
 import logging
 log = logging.getLogger(__name__)
 
@@ -242,6 +242,9 @@ class Status(XacmlContextBase):
 class Decision(object):
     """Define decision types for Response Result
     
+    @cvar ELEMENT_LOCAL_NAME: XML Local Name of StatusCode element 
+    @type ELEMENT_LOCAL_NAME: string
+    
     @cvar PERMIT_STR: permit decision string
     @type PERMIT_STR: string
     
@@ -257,23 +260,53 @@ class Decision(object):
     @cvar TYPES: list of valid decision strings
     @type TYPES: tuple
     
+    @cvar PERMIT: "Permit" decision type instance
+    @type PERMIT: PermitDecision
+    
+    @cvar DENY: "Deny" decision type instance
+    @type DENY: DenyDecision
+    
+    @cvar INDETERMINATE: "Indeterminate" decision type instance
+    @type INDETERMINATE: IndeterminateDecision
+        
+    @cvar NOT_APPLICABLE: "NotApplicable" decision type instance
+    @type NOT_APPLICABLE: NotApplicableDecision
+       
     @ivar __value: decision value
     @type __value: string
     """
+    ELEMENT_LOCAL_NAME  = 'Decision'
     
-    # "Permit" decision type
+    # "Permit" decision type string
     PERMIT_STR = "Permit"
     
-    # "Deny" decision type
+    # "Deny" decision type string
     DENY_STR = "Deny"
     
-    # "Indeterminate" decision type
+    # "Indeterminate" decision type string
     INDETERMINATE_STR = "Indeterminate"
     
+    # "NotApplicable" decision type string
     NOT_APPLICABLE_STR = "NotApplicable"
         
     TYPES = (PERMIT_STR, DENY_STR, INDETERMINATE_STR, NOT_APPLICABLE_STR)
     
+    # "Permit" decision type as an instance of PermitDecision - see 
+    # re-assignment later after definition of PermitDecision class
+    PERMIT = None
+    
+    # "Deny" decision type as instance of DenyDecision - see 
+    # re-assignment later after definition of DenyDecision class
+    DENY = None
+    
+    # "Indeterminate" decision type as instance of IndeterminateDecision - see 
+    # re-assignment later after definition of IndeterminateDecision class
+    INDETERMINATE = None
+        
+    # "NotApplicable" decision type as instance of NotApplicableDecision - see 
+    # re-assignment later after definition of NotApplicableDecision class
+    NOT_APPLICABLE = None
+    
     __slots__ = ('__value',)
     
     def __init__(self, decision=INDETERMINATE_STR):
@@ -608,4 +641,21 @@ class Result(XacmlContextBase):
             raise TypeError('Expecting %r type for result "obligations" '
                             'attribute; got %r' % (Obligation, type(value)))
             
-        self.__obligations = value
\ No newline at end of file
+        self.__obligations = value
+
+    def __getstate__(self):
+        '''Enable pickling
+        
+        @return: object's attribute dictionary
+        @rtype: dict
+        '''
+        _dict = super(Result, self).__getstate__()
+        for attrName in Result.__slots__:
+            # Ugly hack to allow for derived classes setting private member
+            # variables
+            if attrName.startswith('__'):
+                attrName = "_Result" + attrName
+                
+            _dict[attrName] = getattr(self, attrName)
+            
+        return _dict
