@@ -8,7 +8,7 @@ __copyright__ = "(C) 2010 Science and Technology Facilities Council"
 __contact__ = "Philip.Kershaw@stfc.ac.uk"
 __license__ = "BSD - see LICENSE file in top-level directory"
 __contact__ = "Philip.Kershaw@stfc.ac.uk"
-__revision__ = "$Id: __init__.py 7682 2010-11-01 16:30:22Z pjkersha $"
+__revision__ = "$Id: __init__.py 8045 2012-03-16 17:50:04Z rwilkinson $"
 from abc import ABCMeta, abstractmethod
 from datetime import datetime, timedelta
 import traceback
@@ -21,6 +21,12 @@ from ndg.xacml.utils import VettedDict, _isIterable
 from ndg.xacml.utils.factory import callModuleObject
 
 
+# Mapping for function name prefixes that are not real types, but play a similar
+# role
+SPECIAL_TYPE_MAP = {
+    'url-string': 'AnyURI',
+    'xpath-node': 'String'}
+
 class AbstractFunction(object):
     """Abstract Base class for all XACML matching functions
     @cvar FUNCTION_NS: namespace for the given function
@@ -55,6 +61,7 @@ class AbstractFunction(object):
         @return: derived type should return True for match, False otherwise
         @rtype: bool (derived type), NoneType for THIS implementation
         """
+    
         
 class XacmlFunctionNames(object):
     """XACML standard match function names
@@ -312,6 +319,11 @@ def unsupportedFunctionErrorFactory(identifier, msg=None):
             
         raise UnsupportedFunctionError(msg)
     
+
+class OverwritingStdFunctionError(XacmlError):
+    """Attempting to overwrite a standard function namespace with a custom one
+    (probably from load_custom_function method)"""
+    
     
 class FunctionClassFactoryInterface(object):
     """Interface class for function module class factory class
@@ -452,9 +464,11 @@ class FunctionClassFactoryBase(FunctionClassFactoryInterface):
             
         typeURI = AttributeValue.TYPE_URI_MAP.get(typeName)
         if typeURI is None:
-            # Ugly hack to allow for XPath node functions
-            if typePart == 'xpath-node':
-                typeURI = AttributeValue.TYPE_URI_MAP['String']
+            # Ugly hack to allow for functions that start with a prefix that
+            # isn't a real type.
+            if typePart in SPECIAL_TYPE_MAP:
+                typeURI = AttributeValue.TYPE_URI_MAP[
+                                                    SPECIAL_TYPE_MAP[typePart]]
             else:
                 raise TypeError('No AttributeValue.TYPE_URI_MAP entry for '
                                 '%r type' % typePart) 
@@ -561,6 +575,7 @@ class FunctionMap(VettedDict):
         # class factory which creates the various 
         # urn:oasis:names:tc:xacml:1.0:function:<type>-equal function classes
         self.__classFactoryMap = {}
+        self.__custom_class_factory_map = {}
         
     @staticmethod
     def keyFilter(key):
@@ -635,13 +650,21 @@ class FunctionMap(VettedDict):
                 
                 if len(functionNameParts) == 1:
                     moduleName = functionNameParts[0]
-                    
-                elif functionName.startswith('xpath-node'):
-                    # Ugly hack for xpath-node functions
-                    moduleName = functionNameParts[-1].lower()
                 else:
-                    moduleName = '_'.join(functionNameParts[1:]).lower()
-                    
+                    prefix = None
+                    # Ugly hack to allow for functions that start with a prefix
+                    # that isn't a real type.
+                    for pfx in SPECIAL_TYPE_MAP.iterkeys():
+                        pfxsep = pfx + '-'
+                        if functionName.startswith(pfxsep):
+                            prefix = pfxsep
+                            break
+                    if prefix:
+                        suffix = functionName[len(prefix):]
+                        moduleName = '_'.join(suffix.split('-')).lower()
+                    else:
+                        moduleName = '_'.join(functionNameParts[1:]).lower()
+
                 classPath = pkgNamePrefix + moduleName + '.' + \
                             cls.FUNCTION_CLASS_FACTORY_CLASSNAME
                 break
@@ -667,7 +690,39 @@ class FunctionMap(VettedDict):
                 
             self[functionNs] = function
             self.__classFactoryMap[functionNs] = functionFactory
-                       
+    
+    def load_custom_function(self, 
+                             function_ns, 
+                             function_factory=None,
+                             function_factory_path=None):
+        """Add a user defined function to the list of functions supported"""
+        
+        if function_ns in XacmlFunctionNames.FUNCTION_NAMES:
+            raise OverwritingStdFunctionError("Attempting to overwrite the "
+                                              "standard function namespace %r"
+                                              "with a new custom function" %
+                                              function_ns)
+        if function_factory is None:
+            if not isinstance(function_factory_path, basestring):
+                raise TypeError('Expecting "function_factory_path" keyword '
+                                'set to string function factory path; got %r' %
+                                function_factory_path)
+            try:
+                function_factory = callModuleObject(function_factory_path)
+                          
+            except (ImportError, AttributeError), e:
+                log.error("Error importing function factory class %r for custom "
+                          "function identifier %r: %s", function_factory_path, 
+                          function_ns, str(e))
+                raise
+        
+        function = function_factory(function_ns)
+        if function is None:
+            raise unsupportedFunctionErrorFactory(function_ns)
+        
+        self[function_ns] = function
+        self.__custom_class_factory_map[function_ns] = function_factory
+         
     def __getitem__(self, key):
         """Override base class implementation to load and cache function classes
         if they don't otherwise exist
@@ -705,4 +760,3 @@ class FunctionMap(VettedDict):
 # Function map singleton used by match and apply classes - add new keys to
 # this dictionary to enable support for custom functions
 functionMap = FunctionMap()
-        
\ No newline at end of file
