@@ -7,10 +7,9 @@ __date__ = "26/03/10"
 __copyright__ = ""
 __license__ = "BSD - see LICENSE file in top-level directory"
 __contact__ = "Philip.Kershaw@stfc.ac.uk"
-__revision__ = '$Id: regexp_match.py 7351 2010-08-20 14:46:35Z pjkersha $'
+__revision__ = '$Id: regexp_match.py 8017 2012-02-16 08:39:08Z pjkersha $'
 import re
 
-from ndg.xacml.core.context.exceptions import XacmlContextTypeError
 from ndg.xacml.core.attributevalue import AttributeValueClassFactory
 from ndg.xacml.core.functions import (AbstractFunction, 
                                       FunctionClassFactoryInterface)
@@ -33,11 +32,16 @@ class RegexpMatchBase(AbstractFunction):
     
     @cvar CLASS_NAME_SUFFIX: suffix for all regular expression class names
     @type CLASS_NAME_SUFFIX: string
+    
+    @cvar compiled_regexes: cache of compiled regular expressions
+    @type compiled_regexes: dict of string mappings to re.RegexObject
     """
     FUNCTION_NS = None
     FUNCTION_NS_SUFFIX = '-regexp-match'
     CLASS_NAME_SUFFIX = 'RegexpMatch'
     TYPE = None
+
+    compiled_regexes = {}
     
     def evaluate(self, pat, input):
         """Match URI against regular expression pattern
@@ -57,7 +61,11 @@ class RegexpMatchBase(AbstractFunction):
             raise TypeError('Expecting %r derived type for "input"; got %r' %
                             (self.__class__.TYPE, type(input)))
             
-        return bool(re.match(pat.value, input.value))
+        compiled_regex = self.compiled_regexes.get(pat.value, None)
+        if compiled_regex is None:
+            compiled_regex = re.compile(pat.value)
+            self.compiled_regexes[pat.value] = compiled_regex
+        return bool(compiled_regex.match(input.value))
     
 
 attributeValueClassFactory = AttributeValueClassFactory()
