@@ -8,7 +8,7 @@ __copyright__ = "(C) 2010 Science and Technology Facilities Council"
 __contact__ = "Philip.Kershaw@stfc.ac.uk"
 __license__ = "BSD - see LICENSE file in top-level directory"
 __contact__ = "Philip.Kershaw@stfc.ac.uk"
-__revision__ = "$Id: attributevaluereader.py 7661 2010-10-27 15:05:18Z pjkersha $"
+__revision__ = "$Id: attributevaluereader.py 7955 2011-12-21 18:29:45Z rwilkinson $"
 from ndg.xacml.core.attributevalue import (AttributeValue, 
                                            AttributeValueClassFactory)
 from ndg.xacml.parsers import XMLParseError
@@ -30,7 +30,7 @@ class AttributeValueReader(ExpressionReader):
     TYPE = AttributeValue
     FACTORY = AttributeValueClassFactory()
     
-    def __call__(self, obj):
+    def __call__(self, obj, common):
         """Parse *AttributeValue type element - override this method instead of
         _parseExtension since AttributeValue class is virtual.  A sub-type can
         be instantiated only once the data type attribute is parsed
@@ -160,4 +160,4 @@ class DataTypeReaderClassFactory(object):
     @classmethod
     def getReader(cls, attributeValue):
         return cls.MAP[attributeValue.dataType]
-    
\ No newline at end of file
+    
